package datetime

import (
	"context"

	"github.com/MontFerret/ferret/pkg/runtime/core"
	"github.com/MontFerret/ferret/pkg/runtime/values"
)

// DateMonth returns the month of date as a number.
// @params date (DateTime) - source DateTime.
// @return (Int) - a month number.
func DateMonth(_ context.Context, args ...core.Value) (core.Value, error) {
	err := core.ValidateArgs(args, 1, 1)
	if err != nil {
		return values.None, err
	}

	err = core.ValidateType(args[0], core.DateTimeType)
	if err != nil {
		return values.None, err
	}

	month := args[0].(values.DateTime).Month()

	return values.NewInt(int(month)), nil
}
